package de.hsh.dbs2.imdb.logic;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import de.hsh.dbs2.imdb.logic.dto.*;
import de.hsh.dbs2.imdb.records.ConnectionManager;
import de.hsh.dbs2.imdb.records.Genre;
import de.hsh.dbs2.imdb.records.Movie;
import de.hsh.dbs2.imdb.records.MovieCharacter;
import de.hsh.dbs2.imdb.records.MovieGenre;
import de.hsh.dbs2.imdb.records.Person;

public class MovieManager {

	/**
	 * Ermittelt alle Filme, deren Filmtitel den Suchstring enthaelt.
	 * Wenn der String leer ist, sollen alle Filme zurueckgegeben werden.
	 * Der Suchstring soll ohne Ruecksicht auf Gross/Kleinschreibung verarbeitet werden.
	 * @param search Suchstring. 
	 * @return Liste aller passenden Filme als MovieDTO
	 * @throws Exception
	 */
	public List<MovieDTO> getMovieList(String search) throws Exception {
		try {
			ArrayList<MovieDTO> movies = new ArrayList<MovieDTO>();
			for (Movie movie : Movie.findAll()) {
				if (movie.getTitle().toLowerCase().indexOf(search.toLowerCase()) == -1) continue;
				
				MovieDTO moviedto = new MovieDTO();
				
				moviedto.setId(movie.getMovieID());
				moviedto.setTitle(movie.getTitle());
				moviedto.setType("" + movie.getType());
				moviedto.setYear(movie.getYear());
				
				for (MovieGenre mg : MovieGenre.findByMovieID(movie.getMovieID())) {
					moviedto.addGenre(Genre.findById(mg.getGenreID()).getGenre());
				}
				
				for (MovieCharacter mc : MovieCharacter.findByMovieID(movie.getMovieID())) {
					CharacterDTO characterdto = new CharacterDTO();
					
					characterdto.setCharacter(mc.getCharacter());
					characterdto.setAlias(mc.getAlias());
					characterdto.setPlayer(Person.findById(mc.getPlayedBy()).getName());
					
					moviedto.addCharacter(characterdto);
				}
				
				movies.add(moviedto);
			}
			
			ConnectionManager.commit();
			return movies;
		} catch (SQLException e) {
			ConnectionManager.rollback();
			throw e;
		}
			
	}

	/**
	 * Speichert die uebergebene Version des Films neu in der Datenbank oder aktualisiert den
	 * existierenden Film.
	 * Dazu werden die Daten des Films selbst (Titel, Jahr, Typ) beruecksichtigt,
	 * aber auch alle Genres, die dem Film zugeordnet sind und die Liste der Charaktere
	 * auf den neuen Stand gebracht.
	 * @param movie Film-Objekt mit Genres und Charakteren.
	 * @throws Exception
	 */
	public void insertUpdateMovie(MovieDTO movieDTO) throws Exception {
		try {
			Movie movie = new Movie(
				movieDTO.getId(),
				movieDTO.getTitle(),
				movieDTO.getYear(),
				movieDTO.getType().charAt(0)
			);
			if (movie.getMovieID() == null) {
				movie.insert();
			} else {
				movie.update();
			}
			for (MovieGenre genre : MovieGenre.findByMovieID(movie.getMovieID())) {
				genre.delete();
			}
			for (String genreName : movieDTO.getGenres()) {
				Genre genre = Genre.findByGenre(genreName);
				if (genre == null) {
					genre = new Genre(null, genreName);
					genre.insert();
				}
				
				MovieGenre mg = new MovieGenre(movie.getMovieID(), genre.getGenreID());
				mg.insert();
			}
			for (MovieCharacter character : MovieCharacter.findByMovieID(movie.getMovieID())) {
				character.delete();
			}
			int position = 0;
			for (CharacterDTO characterDTO : movieDTO.getCharacters()) {
				Person player = Person.findByName(characterDTO.getPlayer());
				if (player == null) {
					player = new Person(null, characterDTO.getPlayer(), 'U');
				}
				MovieCharacter mc = new MovieCharacter(null, characterDTO.getCharacter(), characterDTO.getAlias(), position, player.getPersonID(), movie.getMovieID());
				mc.insert();
				++position;
			}
			
			ConnectionManager.commit();
			
		} catch (SQLException e) {
			ConnectionManager.rollback();
			throw e;
		}
		
	}

	/**
	 * Loescht einen Film aus der Datenbank. Es werden auch alle abhaengigen Objekte geloescht,
	 * d.h. alle Charaktere und alle Genre-Zuordnungen.
	 * @param movie
	 * @throws Exception
	 */
	public void deleteMovie(long movieId) throws Exception {
		try {			
			Movie.findById(movieId).delete();
			ConnectionManager.commit();
			
		} catch (SQLException e) {
			ConnectionManager.rollback();
			throw e;
		}
	}

	/**
	 * Liefert die Daten eines einzelnen Movies zurück
	 * @param movieId
	 * @return
	 * @throws Exception
	 */
	public MovieDTO getMovie(long movieId) throws Exception {
		try {
			Movie movie = Movie.findById(movieId);
			if (movie == null) {
				throw new Exception("Movie not found");
			}
			MovieDTO moviedto = new MovieDTO();
			
			moviedto.setId(movie.getMovieID());
			moviedto.setTitle(movie.getTitle());
			moviedto.setType("" + movie.getType());
			moviedto.setYear(movie.getYear());
			
			for (MovieGenre mg : MovieGenre.findByMovieID(movie.getMovieID())) {
				moviedto.addGenre(Genre.findById(mg.getGenreID()).getGenre());
			}
			
			for (MovieCharacter mc : MovieCharacter.findByMovieID(movie.getMovieID())) {
				CharacterDTO characterdto = new CharacterDTO();
				
				characterdto.setCharacter(mc.getCharacter());
				characterdto.setAlias(mc.getAlias());
				characterdto.setPlayer(Person.findById(mc.getPlayedBy()).getName());
				
				moviedto.addCharacter(characterdto);
			}
			
			ConnectionManager.commit();
			return moviedto;
			
		} catch (SQLException e) {
			ConnectionManager.rollback();
			throw e;
		}
	}

}
